"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const googleapis_1 = require("googleapis");
const dotenv_1 = require("dotenv");
const moment_timezone_1 = __importDefault(require("moment-timezone"));
dotenv_1.config();
const oauth2Client = new googleapis_1.google.auth.OAuth2(process.env.GOOGLE_CALENDAR_CLIENT_ID, process.env.GOOGLE_CALENDAR_CLIENT_SECRET, process.env.GOOGLE_CALENDAR_REDIRECT_URL ??
    "https://developers.google.com/oauthplayground");
const calendar = googleapis_1.google.calendar("v3");
/*
 * Split an array in equal chunks
 * @source https://stackoverflow.com/a/51514813/1656944
 */
const chunkArrayInGroups = (arr, parts) => {
    const arrayParts = [];
    for (let i = parts; i > 0; i--)
        arrayParts.push(arr.splice(0, Math.ceil(arr.length / i)));
    return arrayParts;
};
/*
 * Get a random element from this array
 */
const randomItemFromArray = (arr) => arr[Math.floor(Math.random() * arr.length)];
/*
 * Live logging for debugging
 */
const log = (params, ...args) => {
    if (params.log) {
        if (typeof params.logger === "function")
            params.logger(...args);
        else
            console.log(args);
    }
};
/*
 * Find a user's events from a single calendar
 * Defaults to their primary calendar
 */
exports.getEventsFromSingleCalendar = async ({ user, from, to, calendarId, auth, }) => {
    oauth2Client.setCredentials({
        access_token: user?.accessToken ?? process.env.GOOGLE_CALENDAR_ACCESS,
        refresh_token: user?.refreshToken ?? process.env.GOOGLE_CALENDAR_REFRESH,
    });
    return ((await calendar.events.list({
        timeMin: from ? moment_timezone_1.default(from).toISOString() : undefined,
        timeMax: to ? moment_timezone_1.default(to).toISOString() : undefined,
        auth: auth ?? oauth2Client,
        calendarId: calendarId ?? "primary",
        maxResults: 3,
    })).data.items || []);
};
/*
 * Get a list of user's events from all calendars
 */
exports.getEventsFromAllCalendars = async (params) => {
    oauth2Client.setCredentials({
        access_token: params.user?.accessToken ?? process.env.GOOGLE_CALENDAR_ACCESS,
        refresh_token: params.user?.refreshToken ?? process.env.GOOGLE_CALENDAR_REFRESH,
    });
    const auth = params.auth ?? oauth2Client;
    const calendars = await calendar.calendarList.list({ auth });
    const allEvents = [];
    for await (const calendar of calendars.data.items ?? []) {
        if (calendar.id) {
            const events = await exports.getEventsFromSingleCalendar({
                ...params,
                calendarId: calendar.id,
            });
            allEvents.push(...events);
        }
    }
    return allEvents;
};
/*
 * List all free slots for a user
 */
exports.getSlots = async (params) => {
    // Default slot duration is 30 minutes
    const slotDuration = params.slotDuration ?? 30;
    const daysAllowed = params.days ?? [0, 1, 2, 3, 4, 5, 6];
    delete params.slotDuration;
    // Find all slots
    const allPotentialSlots = [];
    const differenceInMinutes = moment_timezone_1.default(params.to).diff(params.from, "minutes");
    let endDate = moment_timezone_1.default(params.from).hours(0).minutes(0).seconds(0);
    while (moment_timezone_1.default(endDate).isBefore(params.to)) {
        const start = endDate;
        const end = moment_timezone_1.default(start).add(slotDuration, "minutes");
        if (daysAllowed.includes(moment_timezone_1.default(start).day()) &&
            daysAllowed.includes(end.day()) &&
            start.isAfter(moment_timezone_1.default(params.from)
                .hours(params.daily?.from[0] ?? 0)
                .minutes(params.daily?.from[1] ?? 0)
                .seconds(params.daily?.from[2] ?? 0)) &&
            end.isBefore(moment_timezone_1.default(params.to)
                .hours(params.daily?.to[0] ?? 0)
                .minutes(params.daily?.to[1] ?? 0)
                .seconds(params.daily?.to[2] ?? 0))) {
            const startTime = moment_timezone_1.default(start);
            const endTime = moment_timezone_1.default(end);
            let dailyConditionsMet = true;
            if (params.daily)
                if (moment_timezone_1.default.tz(startTime, params.daily.timezone).isBefore(moment_timezone_1.default
                    .tz(startTime, params.daily.timezone)
                    .hours(params.daily.from[0])
                    .minutes(params.daily.from[1] ?? 0)
                    .seconds(params.daily.from[2] ?? 0)) ||
                    moment_timezone_1.default.tz(endTime, params.daily.timezone).isAfter(moment_timezone_1.default
                        .tz(endTime, params.daily.timezone)
                        .hours(params.daily.to[0])
                        .minutes(params.daily.to[1] ?? 0)
                        .seconds(params.daily.to[2] ?? 0)))
                    dailyConditionsMet = false;
            if (dailyConditionsMet)
                allPotentialSlots.push({
                    start: startTime.toDate(),
                    end: endTime.toDate(),
                });
        }
        endDate = end;
    }
    log(params, `Total potential slots are ${allPotentialSlots.length}`);
    if (!allPotentialSlots.length)
        return [];
    let calendarEvents = [];
    // let timer = new Date().getTime();
    // if (params.calendarId)
    //   calendarEvents = await getEventsFromSingleCalendar(params);
    // else calendarEvents = await getEventsFromAllCalendars(params);
    // log(
    //   params,
    //   `Fetched ${calendarEvents.length} events from ${
    //     params.calendarId ?? "all calendars"
    //   } in ${(new Date().getTime() - timer) / 1000} seconds`
    // );
    const recommendedSlots = allPotentialSlots.filter((slot) => {
        let conflict = false;
        calendarEvents.forEach((event) => {
            if (event.start?.dateTime &&
                event.end?.dateTime &&
                moment_timezone_1.default(slot.start).isAfter(event.start.dateTime) &&
                moment_timezone_1.default(slot.end).isBefore(event.end.dateTime))
                conflict = true;
        });
        return !conflict;
    });
    if (params.slots) {
        if (recommendedSlots.length <= params.slots)
            return recommendedSlots;
        const parts = chunkArrayInGroups(recommendedSlots, params.slots);
        const randomSlots = [];
        for (let i = 0; i < params.slots; i++)
            randomSlots.push(randomItemFromArray(parts[i]));
        log(params, `Recommending ${randomSlots.length} slots`);
        return randomSlots;
    }
    return recommendedSlots;
};
// const test = async () => {
//   const result = await getSlots({
//     slotDuration: 30,
//     log: true,
//     slots: 3,
//     from: new Date(),
//     to: new Date("2020-05-10"),
//     days: [1, 2, 3, 4, 5],
//     daily: {
//       timezone: "Asia/Kolkata",
//       from: [12],
//       to: [16],
//     },
//   });
//   console.log(
//     "RESULT",
//     result.map((i) => i.start.toLocaleString())
//   );
// };
// test();
//# sourceMappingURL=index.js.map